<?php

namespace Drupal\search_api_boolean_field_boost\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\search_api\Plugin\PluginFormTrait;
use Drupal\search_api\Processor\ProcessorPluginBase;
use Drupal\search_api\Utility\Utility;

/**
 * Adds a boost based on a boolean field value.
 *
 * @SearchApiProcessor(
 *   id = "boolean_field_boost",
 *   label = @Translation("Boolean field-based boosting"),
 *   description = @Translation("Adds a boost to indexed items if a selected boolean field is TRUE."),
 *   stages = {
 *     "preprocess_index" = 0,
 *   }
 * )
 */
class BooleanFieldBoost extends ProcessorPluginBase implements PluginFormInterface {

  use PluginFormTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'boosts' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $config = $this->configuration['boosts'];
    $additional_factors = [];
    foreach ($config as $field_configs) {
      if (isset($field_configs['boost_factor'])) {
        $additional_factors[] = $field_configs['boost_factor'];
      }
    }
    $boost_factors = Utility::getBoostFactors($additional_factors);
    $boost_factors[Utility::formatBoostFactor(0)] = $this->t('Ignore');

    foreach ($this->index->getFields(TRUE) as $field_id => $field) {
      // Only include boolean fields.
      if ($field->getType() === 'boolean') {
        $form['boosts'][$field_id] = [
          '#type' => 'details',
          '#title' => $field->getLabel(),
        ];

        $form['boosts'][$field_id]['boost_factor'] = [
          '#type' => 'select',
          '#title' => $this->t('Boost factor'),
          '#options' => $boost_factors,
          '#description' => $this->t('The boost factor to apply if the field value is TRUE. Setting it to "Ignore" means no boost will be applied.'),
          '#default_value' => Utility::formatBoostFactor($config[$field_id]['boost_factor'] ?? 0),
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $ignore = Utility::formatBoostFactor(0);
    foreach ($values['boosts'] as $field_id => $settings) {
      if (!$settings['boost_factor'] || $settings['boost_factor'] === $ignore) {
        unset($values['boosts'][$field_id]);
      }
    }
    $form_state->setValues($values);
    $this->setConfiguration($values);
  }

  /**
   * {@inheritdoc}
   */
  public function preprocessIndexItems(array $items) {
    $boosts = $this->configuration['boosts'];

    /** @var \Drupal\search_api\Item\ItemInterface $item */
    foreach ($items as $item) {
      foreach ($boosts as $field_id => $settings) {
        if ($field = $item->getField($field_id)) {
          if ($values = $field->getValues()) {
            $value = reset($values);
            if ($value) {
              $item->setBoost($item->getBoost() * (double) $settings['boost_factor']);
            }
          }
        }
      }
    }
  }

}
